from gradio_client import Client, handle_file

def recognize_speech_with_gradio(file_name, server_url="http://127.0.0.1:7860/", api_name="/model_inference"):
    """
    使用 Gradio 客户端调用本地语音识别模型，将音频转换为文本
    :param file_name: 要识别的音频文件路径
    :param server_url: Gradio 服务的 URL 地址
    :param api_name: Gradio 服务的 API 名称
    :return: 转录的文本
    """
    print("正在调用 Gradio 语音识别服务...")
    try:
        client = Client(server_url)
        result = client.predict(
            input_wav=handle_file(file_name),
            language="auto",  # 自动检测语言
            api_name=api_name
        )
        if result:
            print(f"识别结果: {result}")
            return result
        else:
            print("未能识别语音内容。")
            return None
    except Exception as e:
        print(f"语音识别服务出错: {e}")
        return None