import requests
from pydub import AudioSegment  # 用于音频合并
import os

# 百度 API 的密钥信息
API_KEY = "4Q2i4BYEzgn6m4Ycomr3Yt0B"
SECRET_KEY = "CW5gHyvmu9pybr4wN9BtRTMmW6V5qr7q"


def text_to_speech(text, file_name="response.wav"):
    """
    将输入的文本通过百度 TTS API 转换为语音，并保存为音频文件。
    如果文本超过 60 个汉字，会按照 60 个汉字切分处理。

    参数:
    - text (str): 要转换为语音的文本内容。
    - file_name (str): 输出音频文件的文件名，默认为 "response.wav"。

    返回:
    - bool: 如果语音生成成功并保存为文件，则返回 True；否则返回 False。
    """
    # 获取百度 TTS API 的 Access Token
    access_token = get_access_token()
    if not access_token:
        print("获取 Access Token 失败！")
        return False

    # 切分文本为不超过 60 个汉字的片段
    text_segments = split_text(text, max_length=60)

    # 临时存储音频片段的文件名
    temp_files = []

    # 遍历每个片段，调用百度 TTS API 生成语音
    for i, segment in enumerate(text_segments):
        print(f"正在处理第 {i + 1}/{len(text_segments)} 个片段：{segment}")
        temp_file = f"temp_segment_{i}.mp3"
        if generate_audio(segment, access_token, temp_file):
            temp_files.append(temp_file)
        else:
            print(f"片段 {i + 1} 语音合成失败！")
            return False

    # 合并所有音频片段
    if merge_audio(temp_files, file_name):
        print(f"语音合成完成，音频文件保存为：{file_name}")
        # 删除临时文件
        for temp_file in temp_files:
            os.remove(temp_file)
        return True
    else:
        print("音频合并失败！")
        return False


def get_access_token():
    """
    使用 API_KEY 和 SECRET_KEY 获取百度 TTS API 的 Access Token。

    返回:
    - str: Access Token，如果获取失败则返回 None。
    """
    url = "https://aip.baidubce.com/oauth/2.0/token"
    params = {
        "grant_type": "client_credentials",
        "client_id": API_KEY,
        "client_secret": SECRET_KEY
    }

    try:
        response = requests.post(url, params=params)
        response_json = response.json()
        return response_json.get("access_token")
    except Exception as e:
        print(f"获取 Access Token 时发生错误：{e}")
        return None


def split_text(text, max_length=60):
    """
    将文本按照指定的最大长度切分为多个片段。

    参数:
    - text (str): 要切分的文本。
    - max_length (int): 每个片段的最大长度。

    返回:
    - List[str]: 切分后的文本片段列表。
    """
    return [text[i:i + max_length] for i in range(0, len(text), max_length)]


def generate_audio(text, access_token, file_name):
    """
    使用百度 TTS API 将文本转换为语音并保存为音频文件。

    参数:
    - text (str): 要转换为语音的文本内容。
    - access_token (str): 百度 API 的 Access Token。
    - file_name (str): 输出音频文件的文件名。

    返回:
    - bool: 如果语音合成成功并保存为文件，则返回 True；否则返回 False。
    """
    # 百度 TTS API 的 URL
    url = "https://tsn.baidu.com/text2audio"

    # 构造请求参数
    params = {
        "tex": text,
        "tok": access_token,
        "cuid": "123456PYTHON",
        "ctp": 1,
        "lan": "zh",
        "spd": 5,
        "pit": 5,
        "vol": 5,
        "per": 103,
        "aue": 3
    }

    # 发送 POST 请求获取语音数据
    headers = {"Content-Type": "application/x-www-form-urlencoded"}
    response = requests.post(url, data=params, headers=headers)

    # 检查请求是否成功
    if response.headers.get("Content-Type") == "audio/mp3" or response.headers.get("Content-Type") == "audio/wav":
        # 将返回的音频数据写入文件
        with open(file_name, "wb") as audio_file:
            audio_file.write(response.content)
        return True
    else:
        # 如果返回的不是音频，而是错误信息，则打印错误
        print(f"语音合成失败，错误信息：{response.text}")
        return False


def merge_audio(audio_files, output_file):
    """
    合并多个音频文件为一个音频文件。

    参数:
    - audio_files (List[str]): 要合并的音频文件列表。
    - output_file (str): 合并后的输出音频文件名。

    返回:
    - bool: 如果合并成功，则返回 True；否则返回 False。
    """
    try:
        # 加载第一个音频文件
        combined = AudioSegment.from_file(audio_files[0])

        # 依次合并剩余的音频文件
        for audio_file in audio_files[1:]:
            segment = AudioSegment.from_file(audio_file)
            combined += segment

        # 导出合并后的音频文件
        combined.export(output_file, format="wav")
        return True
    except Exception as e:
        print(f"音频合并时发生错误：{e}")
        return False


if __name__ == "__main__":
    # 示例：将文本转换为语音并保存为文件
    text = "你好，这是一段测试文本。由于百度语音合成接口的限制，单次最多只能处理六十个汉字。所以我们需要将文本切分成多个片段，然后分别合成语音，最后合并成一个音频文件。"
    output_file = "response.wav"
    success = text_to_speech(text, output_file)
    if success:
        print("语音合成完成！")
    else:
        print("语音合成失败！")
