from GPT import call_model_api
import datetime


def process_user_input(user_input, api_url):
    """
    处理用户输入，根据关键词返回对应的响应。
    :param user_input: 用户输入的文本
    :param api_url: 大模型 API 的地址
    :return: 响应文本
    """
    # 定义关键词及其优先级（数字越小优先级越高）
    keyword_actions = {
        # 控制关键词
        "开始": {"priority": 0, "action": start_recording},
        "结束": {"priority": 0, "action": stop_recording},

        # 内容关键词
        "天气": {"priority": 1, "action": get_weather},
        "时间": {"priority": 2, "action": get_time},

        # 大模型关键词
        "生成报告": {"priority": 3, "action": lambda: call_model_api("生成报告", api_url)},
        "写文章": {"priority": 3, "action": lambda: call_model_api("写文章", api_url)},
    }

    # 检查用户输入中是否包含关键词
    for keyword, config in sorted(keyword_actions.items(), key=lambda x: x[1]["priority"]):
        if keyword in user_input:
            # 调用对应的功能函数
            return config["action"]()

    # 如果没有匹配到关键词，则调用大模型
    print("未匹配到特定关键词，调用大模型生成响应...")
    return call_model_api(user_input, api_url)


# 控制关键词逻辑
def start_recording():
    """
    处理 '开始' 的逻辑：返回提示信息，进入录音模式。
    """
    return "已进入录音模式，请开始说话。"


def stop_recording():
    """
    处理 '结束' 的逻辑：返回提示信息，停止录音。
    """
    return "录音结束，谢谢您的使用。"


# 内容关键词逻辑
def get_weather():
    """
    模拟获取天气信息的功能
    :return: 天气信息
    """
    return "今天的天气是晴天，温度25摄氏度。"


def get_time():
    """
    获取当前时间
    :return: 当前时间信息
    """
    current_time = datetime.datetime.now().strftime("%Y年%m月%d日 %H:%M:%S")
    return f"现在的时间是：{current_time}"
